package com.bwie.gateway.filters;

import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import io.jsonwebtoken.Claims;
import lombok.extern.log4j.Log4j2;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.concurrent.TimeUnit;

@Component
@Log4j2
public class AuthFilter implements GlobalFilter, Ordered {
    private final RedisCache redisCache;
    private final IgnoreWhiteConfig ignoreWhiteConfig;

    public AuthFilter(RedisCache redisCache, IgnoreWhiteConfig ignoreWhiteConfig) {
        this.redisCache = redisCache;
        this.ignoreWhiteConfig = ignoreWhiteConfig;
    }

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpRequest request = exchange.getRequest();
        HttpHeaders headers = request.getHeaders();
        HttpMethod method = request.getMethod();
        ServerHttpRequest.Builder mutate = request.mutate();
        String path = request.getURI().getPath();
        System.out.println("path = " + path);
        System.out.println("method = " + method);
        if(StringUtils.matches(path,ignoreWhiteConfig.getWhites())){
            return chain.filter(exchange);
        }
        String first = headers.getFirst(TokenConstants.TOKEN);
        if(StringUtils.isEmpty(first)){
            return GatewayUtils.errorResponse(exchange,"令牌不能为空");
        }
        Claims claims = JwtUtils.parseToken(first);
        if(claims==null){
            return GatewayUtils.errorResponse(exchange,"令牌已过期或验证不正确");
        }
        String userKey = JwtUtils.getUserKey(claims);
        Boolean b = redisCache.hasKey(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        if(!b){
            return GatewayUtils.errorResponse(exchange,"登陆状态已过期");
        }
        String userId = JwtUtils.getUserId(claims);
        String userName = JwtUtils.getUserName(claims);
        GatewayUtils.addHander(mutate, JwtConstants.USER_KEY,userKey);
        GatewayUtils.addHander(mutate,JwtConstants.DETAILS_USERNAME,userName);
        GatewayUtils.addHander(mutate,JwtConstants.DETAILS_USER_ID,userId);
        GatewayUtils.removeHeader(mutate,TokenConstants.TOKEN);

        return chain.filter(exchange.mutate().request(mutate.build()).build());
    }
//@Override
//public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
//    List<String> whites = ignoreWhiteConfig.getWhites();
//    String path = exchange.getRequest().getURI().getPath();
//    System.out.println("path >>>>>>>>>>>>>>>>>>>>>>>> = " + path);
//    if(StringUtils.matches(path,whites)){
//        return chain.filter(exchange);
//    }
//    String first = exchange.getRequest().getHeaders().getFirst(TokenConstants.TOKEN);
//    if(null==first){
//        return GatewayUtils.errorResponse(exchange,"token不为空");
//    }
//    try {
//        JwtUtils.parseToken(first);
//    } catch (Exception e) {
//        return GatewayUtils.errorResponse(exchange,"token格式不正确");
//    }
//    String userKey = JwtUtils.getUserKey(first);
//    if(!redisCache.hasKey(TokenConstants.LOGIN_TOKEN_KEY+userKey)){
//        return GatewayUtils.errorResponse(exchange,"token已过期");
//    }
//
//    redisCache.expire(TokenConstants.LOGIN_TOKEN_KEY+userKey,30, TimeUnit.MINUTES);
//    return chain.filter(exchange);
//}


    @Override
    public int getOrder() {
        return 0;
    }


}
